@@ -22,12 +22,12 @@
 #   PA.strategy <- Pseudo Absences strategy
 #   PA.dist.min <- If strategy is 'disk' : Pseudo Absences minimum distance between pres and selected absences (in metters if explanatory is georeferenced or in resp.xy units in all other cases)
 #   PA.dist.man <- If strategy is 'disk' : Pseudo Absences maximum distance between pres and selected absences (in metters if explanatory is georeferenced or in resp.xy units in all other cases)
-# 
+#
 #   PA.sre.quant <- If strategy is 'sre' : the quantile use for sre calculation
 #   PA.table <- If strategy is 'user.defined' : a boolean data.frame indiacating which points of resp.var should be sonsidered in each PA run.
 #   na.rm <- if True na are automatically removed
 
-# OUTPUT : 
+# OUTPUT :
 #   a BIOMOD.formated.data object that will be given to BIOMOD_Modeling function
 
 ####################################################################################################
@@ -48,7 +48,7 @@
                                    PA.table = NULL,
                                    na.rm = TRUE){
   .bmCat(paste(resp.name, " Data Formating", sep=""))
-  
+
   # 1 check args
   args <- .BIOMOD_FormatingData.check.args(resp.var,
                                            expl.var,
@@ -64,15 +64,13 @@
                                            PA.dist.max,
                                            PA.sre.quant,
                                            PA.table)
-  
-  cat("\n*** data checking ok")
-  
+
   resp.var <- args$resp.var
-  expl.var <- args$expl.var 
+  expl.var <- args$expl.var
   resp.xy <- args$resp.xy
   resp.name <- args$resp.name
   eval.resp.var <- args$eval.resp.var
-  eval.expl.var <- args$eval.expl.var 
+  eval.expl.var <- args$eval.expl.var
   eval.resp.xy <- args$eval.resp.xy
   PA.nb.rep <- args$PA.nb.rep
   PA.nb.absences <- args$PA.nb.absences
@@ -81,14 +79,13 @@
   PA.dist.max <- args$PA.dist.max
   PA.sre.quant <- args$PA.sre.quant
   PA.table <- args$PA.table
-  
+
   rm('args')
   gc()
-  
+
   out <- NULL
-  
+
   if(PA.strategy == 'none'){ # no Pseudo Absences
-    cat("\n*** called BIOMOD.formated.data")
     out <- BIOMOD.formated.data(sp=resp.var,
                                 xy=resp.xy,
                                 env=expl.var,
@@ -97,17 +94,16 @@
                                 eval.env=eval.expl.var,
                                 eval.xy=eval.resp.xy,
                                 na.rm=na.rm)
-    cat("\n*** BIOMOD.formated.data done")
   } else{ # Automatic Pseudo Absences Selection
     out <- BIOMOD.formated.data.PA(sp=resp.var, xy=resp.xy, env=expl.var, sp.name=resp.name,
                                    eval.sp=eval.resp.var, eval.env=eval.expl.var, eval.xy=eval.resp.xy,
-                                   PA.NbRep=PA.nb.rep, PA.strategy=PA.strategy, 
+                                   PA.NbRep=PA.nb.rep, PA.strategy=PA.strategy,
                                    PA.nb.absences = PA.nb.absences, PA.dist.min = PA.dist.min,
-                                   PA.dist.max = PA.dist.max, PA.sre.quant = PA.sre.quant, PA.table=PA.table, 
+                                   PA.dist.max = PA.dist.max, PA.sre.quant = PA.sre.quant, PA.table=PA.table,
                                    na.rm=na.rm)
-  } 
-  
-  
+  }
+
+
   .bmCat("Done")
   return(out)
 }
@@ -126,60 +122,60 @@
                                              PA.dist.max,
                                              PA.sre.quant,
                                              PA.table){
-  
+
   # 0. names checking
 
-  
-  
+
+
   ### check resp.name is available
   if(grepl('_',resp.name) | grepl(' ',resp.name)){
     resp.name <- paste(unlist(strsplit(resp.name,'_')),collapse='.')
     resp.name <- paste(unlist(strsplit(resp.name,' ')),collapse='.')
-    
+
     cat('\n Response variable name was converted into', resp.name)
   }
-  
+
   ### check resp.name is available
   ### Not done because no imporance
-  
+
   # 1. Checking input params class
-  available.types <- c( 'numeric', 'data.frame', 'matrix', 
+  available.types <- c( 'numeric', 'data.frame', 'matrix',
                         'RasterLayer', 'RasterStack',
                         'SpatialPointsDataFrame', 'SpatialPoints')
   ###### resp.var
-  if(!(class(resp.var) %in% available.types)){
+  if(!inherits(resp.var, available.types)){
     stop( paste("Response variable must be one of ", toString(available.types), sep="") )
   }
-    
+
   ### response var raster object not supported yet
   if(inherits(resp.var, 'Raster')){
     stop("Raster response variable not supported yet ! \nPlease extract your Presences and your absences by yourself")
     #### TO DO ####
     ## extract the 0 and 1 in sp format
   }
-  
+
   ###### expl.var
-  if(!(class(expl.var) %in% available.types[-which(available.types == 'SpatialPoints')])){
+  if(!inherits(expl.var, setdiff(available.types, 'SpatialPoints'))){
     stop( paste("Explanatory variable must be one of ", toString(available.types), sep="") )
   }
-    
-  
+
+
   ###### resp.xy
   if(inherits(resp.var,'SpatialPoints') ){
     if(!is.null(resp.xy)){
       cat("\n      ! XY coordinates of response variable will be ignored because spatial response object is given.")
     }
     resp.xy <- data.matrix(sp::coordinates(resp.var))
-    if(class(resp.var) == 'SpatialPointsDataFrame'){
+    if(inherits(resp.var, 'SpatialPointsDataFrame')){
       resp.var <- resp.var@data
     } else{
-      cat("\n      ! Response variable is considered as a only presences one... Is it really what you want?")
+      cat("\n      ! Response variable is considered as only presences... Is it really what you want?")
       resp.var <- rep(1,nrow(resp.xy))
     }
-    
+
   }
-  
-  
+
+
   ### transforming into numeric if data.frame or matrix
   if(is.matrix(resp.var) | is.data.frame(resp.var)){
     if(ncol(resp.var) > 1){
@@ -188,20 +184,20 @@
       resp.var <- as.numeric(resp.var[,1])
     }
   }
-  
+
   if(is.matrix(expl.var) | is.numeric(expl.var) ){
     expl.var <- as.data.frame(expl.var)
   }
-      
+
   if(inherits(expl.var, 'Raster')){
     expl.var <- raster::stack(expl.var, RAT=FALSE)
   }
-    
+
   if(inherits(expl.var, 'SpatialPoints')){
     expl.var <- as.data.frame(expl.var@data)
   }
-      
-  ### check of xy coordinates validity    
+
+  ### check of xy coordinates validity
   if(!is.null(resp.xy)){
     if(ncol(resp.xy)!=2){
       stop("if given, resp.xy must be a 2 column matrix or data.frame")
@@ -211,76 +207,76 @@
     }
     resp.xy <- as.data.frame(resp.xy)
   }
-  
+
   ### convert response var into binary
   resp.var[which(resp.var>0)] <- 1
   resp.var[which(resp.var<=0)] <- 0
-      
+
   #### At this point :
   ####  - resp.var is a numeric
   ####  - resp.xy is NULL or a data.frame
   ####  - expl.var is a data.frame or a RasterStack
   ####  - sp.name is a character
-  
+
   ### check resp and expl var compatibility
   if(is.data.frame(expl.var)){
     if(nrow(expl.var) != length(resp.var)){
-      stop("If explanatory variable is not a raster then dimentions of response variable and explanatory variable must match!")
+      stop("If explanatory variable is not a raster then dimensions of response variable and explanatory variable must match!")
     }
   }
-  
+
   ### PA strategy
 #   if(!is.null(PA.strategy)){ # force PA.nb.rep to be positive if PA.strategy is defined
 #     PA.nb.rep = max(c(PA.nb.rep,1))
 #   }
-  
+
   if(is.null(PA.table) & PA.nb.rep < 1){
     cat("\n> No pseudo absences selection !")
     PA.strategy <- "none"
     PA.nb.rep <- 0
   }
-  
+
   if(is.null(PA.strategy) &  PA.nb.rep > 0){
     cat("\n> Pseudo absences will be selected randomly !")
     PA.strategy <- "random"
   }
-  
-  
+
+
   if( !is.null(PA.table)){
     cat("\n> Pseudo absences used will be user defined ones !")
     PA.strategy <- "user.defined"
     PA.nb.rep <- 0
   }
-  
+
   if(PA.strategy == "user.defined"){
     if(! (is.matrix(PA.table) | is.data.frame(PA.table)))
       stop("\n PA.table must be a matrix or a data.frame")
-    
+
     if(nrow(PA.table) != length(resp.var))
-      stop("\n PA.table must have as many row than the number 
+      stop("\n PA.table must have as many row than the number
            of observation of your response variable")
-    
+
     #PA.table <- as.data.frame(sapply(PA.table,simplify=FALSE,as.logical))
     colnames(PA.table) <- paste("PA",1:ncol(PA.table),sep="")
-    
+
   }
-  
+
   # 2. eval.resp.var.checking
-     
+
   if(!is.null(eval.resp.var)){
     # do the same test than previous one
     ###### eval.resp.var
     if(!(class(eval.resp.var) %in% available.types)){
       stop( paste("Response variable must be one of ", toString(available.types), sep="") )
     }
-      
+
     ### response var raster object not supported yet
     if(inherits(eval.resp.var, 'Raster')){
       stop("Raster response variable not supported yet ! \nPlease extract your Presences and your absences by yourself")
       #### TO DO ####
       ## extract the 0 and 1 in sp format
     }
-    
+
     ###### expl.var
     if(!is.null(eval.expl.var)){
       if(!(class(eval.expl.var) %in% available.types[-which(available.types == 'SpatialPoints')])){
@@ -301,13 +297,13 @@
       if(class(eval.resp.var) == 'SpatialPointsDataFrame'){
         eval.resp.var <- eval.resp.var@data
       } else{
-        cat("\n      ! Response variable is considered as a only presences one... Is it really what you want?")
+        cat("\n      ! Response variable is considered as only presences... Is it really what you want?")
         eval.resp.var <- rep(1,nrow(eval.resp.xy))
       }
-      
+
     }
-    
-    
+
+
     ### transforming into numeric if data.frame or matrix
     if(is.matrix(eval.resp.var) | is.data.frame(eval.resp.var)){
       if(ncol(eval.resp.var) > 1){
@@ -316,20 +312,20 @@
         eval.resp.var <- as.numeric(eval.resp.var[,1])
       }
     }
-    
+
     if(is.matrix(eval.expl.var) | is.numeric(eval.expl.var) ){
       eval.expl.var <- as.data.frame(eval.expl.var)
     }
-        
+
     if(inherits(eval.expl.var, 'Raster')){
       eval.expl.var <- raster::stack(eval.expl.var)
     }
-      
+
     if(inherits(eval.expl.var, 'SpatialPoints')){
       eval.expl.var <- as.data.frame(eval.expl.var@data)
     }
-        
-    ### check of xy coordinates validity    
+
+    ### check of xy coordinates validity
     if(!is.null(eval.resp.xy)){
       if(ncol(eval.resp.xy)!=2){
         stop("if given, resp.xy must be a 2 column matrix or data.frame")
@@ -342,13 +338,13 @@
 
     if(is.data.frame(eval.expl.var)){
       if(nrow(eval.expl.var) != length(eval.resp.var)){
-        stop("If explanatory variable is not a raster then dimentions of response variable and explanatory variable must match!")
+        stop("If explanatory variable is not a raster then dimensions of response variable and explanatory variable must match!")
       }
     }
 
     ### remove NAs from evaluation data
     if( sum(is.na(eval.resp.var)) > 0 ){
-      cat("\n      ! NAs have been automaticly removed from Evaluation data")
+      cat("\n      ! NAs have been automatically removed from Evaluation data")
       if(!is.null(eval.resp.xy)){
         eval.resp.xy <- eval.resp.xy[-which(is.na(eval.resp.var)),]
       }
@@ -364,13 +360,13 @@
       stop("Evaluation response data must have both presences and absences")
     }
 
-  } else { 
+  } else {
     cat("\n      ! No data has been set aside for modeling evaluation")
     eval.expl.var <- eval.resp.xy <- NULL
   }
-  
+
   ### PA arguments are not checked here because it will be done later... (may be will do it here later)
-  
+
   return(list( resp.var = resp.var,
                expl.var = expl.var,
                resp.xy = resp.xy,
@@ -385,5 +381,5 @@
                PA.dist.max = PA.dist.max,
                PA.sre.quant = PA.sre.quant,
                PA.table = PA.table))
-  
-}
\ No newline at end of file
+
+}
