@@ -115,6 +115,7 @@
 ##' pres.only.eval <- BIOMOD_presenceonly(myBiomodModelOut, myBiomodEM, bg.env = bg.Values)
 ##' pres.only.eval$eval
 ##' }
+
 BIOMOD_presenceonly <- function(modeling.output = NULL, EM.output = NULL, bg.env = NULL, perc = 0.9, save.output = T){
   # if(!require(PresenceAbsence)){stop("PresenceAbsence package required!")}
   requireNamespace('PresenceAbsence', quietly = TRUE)
@@ -128,8 +129,8 @@ BIOMOD_presenceonly <- function(modeling.output = NULL, EM.output = NULL, bg.env
     myModelEval <- get_evaluations(modeling.output,as.data.frame=T)
     myModelEval[,1] <- as.character(myModelEval[,1])
     for(i in 1:nrow(myModelEval)){myModelEval[i,1] <- paste(c(modeling.output@sp.name,strsplit(as.character(myModelEval[i,1]),split="_")[[1]][3:1]),collapse="_")  } 
-
-        myModelPred <- get_predictions(modeling.output,as.data.frame=T)
+    
+    myModelPred <- get_predictions(modeling.output,as.data.frame=T)
     if(!is.null(bg.env)){
       myModelPred.pres <- myModelPred[myResp==1,]
       myBiomodProj.eval <- BIOMOD_Projection(
@@ -160,9 +161,9 @@ BIOMOD_presenceonly <- function(modeling.output = NULL, EM.output = NULL, bg.env
     if(!is.null(modeling.output)){
       myModelEval <- rbind(myModelEval, myModelEvalEF)
     }
-
-      myBiomodProjFF <- get_predictions(EM.output,as.data.frame=T)  
-
+    
+    myBiomodProjFF <- get_predictions(EM.output,as.data.frame=T)  
+    
     if(!is.null(bg.env)){
       myBiomodProjFF.pres <- as.data.frame(myBiomodProjFF[myResp==1,])
       colnames(myBiomodProjFF.pres) <- colnames(myBiomodProjFF)
@@ -176,9 +177,8 @@ BIOMOD_presenceonly <- function(modeling.output = NULL, EM.output = NULL, bg.env
     myModelPred <- cbind(myModelPred, myBiomodProjFF)
     
     if(modeling.output@has.evaluation.data == T){
-    
-        myBiomodProjFF.eval <- get_predictions(EM.output,as.data.frame=T,evaluation=T)  
-
+      myBiomodProjFF.eval <- get_predictions(EM.output,as.data.frame=T,evaluation=T)  
+      
       #colnames(myBiomodProjFF.eval) <- gsub("AllAlgos_ROC_EMwmean","EF",  colnames(myBiomodProjFF.eval))
       myModelPred.eval <- cbind(myModelPred.eval, myBiomodProjFF.eval)      
     }  
@@ -189,7 +189,6 @@ BIOMOD_presenceonly <- function(modeling.output = NULL, EM.output = NULL, bg.env
   mpa.eval$Eval.metric <- "mpa"
   boyce.eval[,3:7]<-mpa.eval[,3:7]<-NA
   
-
   ###MPA & BOYCE     
   for(i in 1:nrow(boyce.eval)){
     n <- length(strsplit(as.character(boyce.eval[i,1]),split="_")[[1]])
@@ -197,73 +196,73 @@ BIOMOD_presenceonly <- function(modeling.output = NULL, EM.output = NULL, bg.env
     Model.name <- boyce.eval[i,1]
     run <- strsplit(Model.name,split="_")[[1]][c(grep("RUN",strsplit(Model.name,split="_")[[1]]),grep("Full",strsplit(Model.name,split="_")[[1]]))]
     
+    #### CORRECTION ------------------------------------------------------------------------
     if(inherits(calib.lines, "matrix")){
-    if(sum(!calib.lines[,paste("_",run, sep="")])==0){      #this is the full model
+      ind.eval = which(calib.lines[,paste("_",run, sep="")] == FALSE) #### CORRECTION
+    }else{
+      ind.eval = which(calib.lines == FALSE) #### CORRECTION
+    }
+    
+    if(length(ind.eval)==0){      #this is the full model ##### CORRECTION
       if(is.null(bg.env)){
-      test <- myResp      
-      Pred<-myModelPred[,Model.name] 
+        # if(inherits(calib.lines, "matrix")){ #### PROBLEM : this part gives problem with the cbind after (not same number of rows)
+        #   test <- myResp
+        #   Pred<-myModelPred[,Model.name]
+        # }else{
+          test <- myResp[calib.lines]     
+          Pred <- myModelPred[calib.lines,Model.name]
+        # }
       }else{
         test <- c(myResp[myResp==1],rep(0,nrow(bg.env)))    
         Pred <- c(myModelPred.pres[,Model.name],myModelPred[,Model.name])       
       }
     }else{
       if(is.null(bg.env)){
-      test <- myResp[!calib.lines[,paste("_",run, sep="")]]
-      Pred <- myModelPred[!calib.lines[,paste("_",run, sep="")],Model.name] 
+        test <- myResp[ind.eval] #### CORRECTION
+        Pred <- myModelPred[ind.eval,Model.name] #### CORRECTION
       }else{
-        test <- c(myResp[!calib.lines[,paste("_",run, sep="")] & myResp==1],rep(0,nrow(bg.env)))
-        Pred <- c(myModelPred.pres[!calib.lines[,paste("_",run, sep="")] & myResp==1,Model.name],myModelPred[,Model.name]) 
+        test <- c(myResp[ind.eval & myResp==1],rep(0,nrow(bg.env))) #### CORRECTION
+        Pred <- c(myModelPred.pres[ind.eval & myResp==1,Model.name],myModelPred[,Model.name])  #### CORRECTION
       }
     }
-    }else{
-      if(sum(!calib.lines)==0){      #this is the full model
-        if(is.null(bg.env)){
-          test <- myResp[calib.lines]     
-          Pred <- myModelPred[calib.lines,Model.name]  
-        }else{
-        test <- c(myResp[myResp==1],rep(0,nrow(bg.env)))
-        Pred<-c(myModelPred.pres[,Model.name],myModelPred[,Model.name])                        
-        }}else{                 #this is the cv model
-          if(is.null(bg.env)){
-        test <- myResp[!calib.lines]
-        Pred<-myModelPred[!calib.lines,Model.name]      
-          }else{
-            test <- c(myResp[!calib.lines & myResp==1],rep(0,nrow(bg.env)))
-            Pred<-c(myModelPred.pres[!calib.lines[myResp==1],Model.name],myModelPred[,Model.name])             
-          }
-      }      
-    }
+    #### CORRECTION ------------------------------------------------------------------------
     
-    if(sum(!is.na(Pred))>1){
-      boy <- ecospat::ecospat.boyce(fit=Pred[!is.na(Pred)],obs=Pred[test==1 & !is.na(test)], PEplot=F)
+    
+    ind.1 = which(test == 1)
+    ind.notNA = which(!is.na(Pred))
+    ind.obs = intersect(ind.1, ind.notNA)
+    
+    if (length(ind.obs) > 0){ #### CORRECTION
+      boy <- ecospat::ecospat.boyce(fit=Pred[ind.notNA],obs=Pred[ind.obs], PEplot=F) #### CORRECTION
       boyce.eval[boyce.eval[,1]==Model.name,3] <- boy$Spearman.cor
       if( sum(boy$F.ratio<1,na.rm=T)>0){
-        boyce.eval[boyce.eval[,1]==Model.name,5] <-  round(boy$HS[max(which(boy$F.ratio<1))],0)
-        DATA<-cbind(1:length(Pred), test, Pred/1000)
+        boyce.eval[boyce.eval[,1]==Model.name,5] <- round(boy$HS[max(which(boy$F.ratio<1))],0)
+        DATA<-cbind(1:length(Pred), test, Pred/1000) #### PROBLEM
         DATA[is.na(DATA[,2]),2] <- 0
         DATA <- DATA[stats::complete.cases(DATA),]
         if(!is.na(round(boy$HS[max(which(boy$F.ratio<1))],0)/1000)){
-        EVAL<-presence.absence.accuracy(DATA, threshold=round(boy$HS[max(which(boy$F.ratio<1))],0)/1000) 
-        boyce.eval[boyce.eval[,1]==Model.name,6] <-  EVAL$sensitivity 
-        boyce.eval[boyce.eval[,1]==Model.name,7] <-  EVAL$specificity
+          EVAL<-presence.absence.accuracy(DATA, threshold=round(boy$HS[max(which(boy$F.ratio<1))],0)/1000) 
+          boyce.eval[boyce.eval[,1]==Model.name,6] <-  EVAL$sensitivity 
+          boyce.eval[boyce.eval[,1]==Model.name,7] <-  EVAL$specificity
         }else{boyce.eval[boyce.eval[,1]==Model.name,6:7] <-  NA}
       }else{
         boyce.eval[boyce.eval[,1]==Model.name,7] <-  boyce.eval[boyce.eval[,1]==Model.name,6] <-  boyce.eval[boyce.eval[,1]==Model.name,5] <- NA 	
       }
       
-      mpa.eval[mpa.eval[,1]==Model.name,5] <- ecospat::ecospat.mpa(Pred[test==1 & !is.na(test)& !is.na(Pred)], perc = perc)
-      EVAL<-presence.absence.accuracy(DATA, threshold=ecospat::ecospat.mpa(Pred[test & !is.na(test)& !is.na(Pred)], perc = perc)/1000) 
+      mpa.eval[mpa.eval[,1]==Model.name,5] <- ecospat::ecospat.mpa(Pred[ind.obs], perc = perc) #### CORRECTION
+      EVAL<-presence.absence.accuracy(DATA, threshold=ecospat::ecospat.mpa(Pred[ind.obs], perc = perc)/1000) #### CORRECTION
       mpa.eval[mpa.eval[,1]==Model.name,6] <-  EVAL$sensitivity 
       mpa.eval[mpa.eval[,1]==Model.name,7] <-  EVAL$specificity  
     }
     
     if(modeling.output@has.evaluation.data == T){
       myResp.eval <- get(load(modeling.output@formated.input.data@link))@eval.data.species
-      Pred.eval <- myModelPred.eval[,Model.name]   
-      boy <- ecospat::ecospat.boyce(fit=Pred.eval,obs=Pred.eval[myResp.eval==1 & !is.na(test)], PEplot=F)
+      Pred.eval <- myModelPred.eval[,Model.name]
+      
+      boy <- ecospat::ecospat.boyce(fit=Pred.eval,obs=Pred.eval[myResp.eval==1 & ind.1], PEplot=F) #### CORRECTION
       boyce.eval[boyce.eval[,1]==Model.name,"Evaluating.data"] <- boy$Spearman.cor
       
-      mpa.eval[mpa.eval[,1]==Model.name,"Evaluating.data"] <- ecospat::ecospat.mpa(Pred.eval[myResp.eval==1 & !is.na(test)], perc = perc) 
+      mpa.eval[mpa.eval[,1]==Model.name,"Evaluating.data"] <- ecospat::ecospat.mpa(Pred.eval[myResp.eval==1 & ind.1], perc = perc) #### CORRECTION
     }
   }
   myModelEval[,6:7] <- round(myModelEval[,6:7],1)
@@ -272,13 +271,13 @@ BIOMOD_presenceonly <- function(modeling.output = NULL, EM.output = NULL, bg.env
   
   if(modeling.output@has.evaluation.data == T){
     if(!is.null(EM.output)){
-    output <- list(eval=rbind(myModelEval,boyce.eval,mpa.eval),myBiomodProjFF=myBiomodProjFF,myBiomodProjEF.eval=myBiomodProjFF.eval) 
+      output <- list(eval=rbind(myModelEval,boyce.eval,mpa.eval),myBiomodProjFF=myBiomodProjFF,myBiomodProjEF.eval=myBiomodProjFF.eval) 
     }else{
       output <- list(eval=rbind(myModelEval,boyce.eval,mpa.eval))      
     }
     if(save.output){
       if(!is.null(modeling.output)){
-      sp<-modeling.output@sp.name}
+        sp<-modeling.output@sp.name}
       if(!is.null(EM.output)){
         sp<-EM.output@sp.name}      
       save(output, paste(sp, "/.BIOMOD_DATA/",modeling.output@modeling.id,"/presenceonlyevaluation_",sp, sep=""))
@@ -286,7 +285,7 @@ BIOMOD_presenceonly <- function(modeling.output = NULL, EM.output = NULL, bg.env
     return(output)
   }else{
     if(!is.null(EM.output)){
-    output <- list(eval=rbind(myModelEval,boyce.eval,mpa.eval),myBiomodProjFF=myBiomodProjFF)
+      output <- list(eval=rbind(myModelEval,boyce.eval,mpa.eval),myBiomodProjFF=myBiomodProjFF)
     }else{
       output <- list(eval=rbind(myModelEval,boyce.eval,mpa.eval))      
     }
@@ -296,7 +295,7 @@ BIOMOD_presenceonly <- function(modeling.output = NULL, EM.output = NULL, bg.env
       if(!is.null(EM.output)){
         sp<-EM.output@sp.name}      
       save(output, file=paste(sp, "/.BIOMOD_DATA/",modeling.output@modeling.id,"/presenceonlyevaluation_",sp, sep=""))
-      }
-    return(output)
     }
+    return(output)
+  }
 }
